//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <cyg/infra/cyg_type.h>

#include <pkgconf/kernel.h>
#include <cyg/kernel/thread.hxx>
#include <cyg/kernel/thread.inl>
#include <cyg/hal/hal_arch.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/kernel/intr.hxx>
#include <cyg/hal/mpc555_regs.h>
#include <cyg/hal/mpc555_isrsys.h>

//--------------------
// Function prototypes
//--------------------
int main(void);
void test_invoke_main(CYG_ADDRWORD);

//--------------------------------------------------------
// Define the process stack size and stack and main thread
//--------------------------------------------------------
#define TEST_MAIN_STACK_SIZE 4096
cyg_uint8 test_main_stack[TEST_MAIN_STACK_SIZE];
Cyg_Thread test_main_thread(CYG_SCHED_DEFAULT_INFO,
                            &test_invoke_main, (CYG_ADDRWORD)0,
                            "main",
                            (CYG_ADDRESS)&test_main_stack,
                            TEST_MAIN_STACK_SIZE);

//-----------------------
// The hook to the kernel
//-----------------------
externC void 
cyg_user_start(void)
{
  test_main_thread.resume();
}

void
test_invoke_main(CYG_ADDRWORD)
{
  main();
}

//------------------
// The placement new
//------------------
void * operator new(size_t size, void * p)
{
  CYG_UNUSED_PARAM(size_t, size);

  return p;
}

//--------------------
// Function prototypes
//--------------------
static cyg_ISR isr_tb_refA, isr_tb_refB;
static cyg_DSR dsr_tb_refA, dsr_tb_refB;
static cyg_ISR isr_rtc_sec;
static cyg_DSR dsr_rtc_sec;
static cyg_ISR isr_rtc_alr;
static cyg_DSR dsr_rtc_alr;
static cyg_ISR isr_pit;
static cyg_DSR dsr_pit;
static cyg_ISR isr_tx;
static cyg_DSR dsr_tx;

static char tb_refA_obj[sizeof(Cyg_Interrupt)];
static char tb_refB_obj[sizeof(Cyg_Interrupt)];
static char rtc_sec_obj[sizeof(Cyg_Interrupt)];
static char rtc_alr_obj[sizeof(Cyg_Interrupt)];
static char pit_obj[sizeof(Cyg_Interrupt)];
static char tx_obj[sizeof(Cyg_Interrupt)];

//----------------------
// Some global variables
//----------------------
int refA_count = 0;
int refB_count = 0;
int seconds = 0;
unsigned long pit_count = 0;

// Something we can set a breakpoint on
void testfunc(cyg_uint16 * val)
{
  (*val)++;
  if((*val) == 1000)
  {
    (*val) = 0;
    unsigned char c = 'X';
    SC2DR = (short)c;
  }
}

//-------------------
// Interrupt handlers
//-------------------
static cyg_uint32 isr_tb_refA(cyg_vector vector, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  Cyg_Interrupt::mask_interrupt(vector);
  Cyg_Interrupt::acknowledge_interrupt(vector);

  return Cyg_Interrupt::CALL_DSR;
}

static cyg_uint32 isr_tb_refB(cyg_vector vector, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);
  
  Cyg_Interrupt::mask_interrupt(vector);
  Cyg_Interrupt::acknowledge_interrupt(vector);

  CYGARC_CLEARTB;
  return Cyg_Interrupt::CALL_DSR;
}

static cyg_uint32 isr_rtc_sec(cyg_vector vector, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  Cyg_Interrupt::mask_interrupt(vector);
  Cyg_Interrupt::acknowledge_interrupt(vector);

  return Cyg_Interrupt::CALL_DSR;
}

static cyg_uint32 isr_rtc_alr(cyg_vector vector, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  Cyg_Interrupt::mask_interrupt(vector);
  Cyg_Interrupt::acknowledge_interrupt(vector);

  return Cyg_Interrupt::CALL_DSR;
}

static cyg_uint32 isr_pit(cyg_vector vector, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  Cyg_Interrupt::mask_interrupt(vector);
  Cyg_Interrupt::acknowledge_interrupt(vector);

  return Cyg_Interrupt::CALL_DSR;
}

static cyg_uint32 isr_tx(cyg_vector vector, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  Cyg_Interrupt::mask_interrupt(vector);
  Cyg_Interrupt::acknowledge_interrupt(vector);

  return Cyg_Interrupt::CALL_DSR;
}

static void dsr_tb_refA(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(cyg_ucount32, count);
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  refA_count++;
  Cyg_Interrupt::unmask_interrupt(vector);
}

static void dsr_tb_refB(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(cyg_ucount32, count);
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  refB_count++;
  Cyg_Interrupt::unmask_interrupt(vector);
}

static void dsr_rtc_sec(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(cyg_ucount32, count);
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  seconds++;
  Cyg_Interrupt::unmask_interrupt(vector);
}

static void dsr_rtc_alr(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(cyg_ucount32, count);
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_RTC);
  CYGARC_RTCRESET;
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_RTC);
  Cyg_Interrupt::unmask_interrupt(vector);
}

static void dsr_pit(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(cyg_ucount32, count);
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_PITC);
  CYGARC_SETPITCOUNT(0x500);
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_PITC);
  pit_count++;
  Cyg_Interrupt::unmask_interrupt(vector);
}

static void dsr_tx(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD data)
{
  CYG_UNUSED_PARAM(cyg_ucount32, count);
  CYG_UNUSED_PARAM(CYG_ADDRWORD, data);

  Cyg_Interrupt::unmask_interrupt(vector);
}

// Make it link, this is testcode afterall
externC void _exit(int)
{
}

// This is the entry point
int main(void)
{
  // Create the interrupts, generate them on the same priority
  int in_use;
  HAL_INTERRUPT_IN_USE(CYGNUM_HAL_INTERRUPT_SIU_TB_A, in_use);
  Cyg_Interrupt * intrA = NULL;
  if(!in_use)
    intrA = new((void *)&tb_refA_obj[0]) Cyg_Interrupt(CYGNUM_HAL_INTERRUPT_SIU_TB_A,
                                                       CYGNUM_HAL_INTERRUPT_SIU_TB_A_PRIORITY,
                                                       (CYG_ADDRWORD)111,
                                                       &isr_tb_refA,
                                                       &dsr_tb_refA);

  HAL_INTERRUPT_IN_USE(CYGNUM_HAL_INTERRUPT_SIU_TB_B, in_use);
  Cyg_Interrupt * intrB = NULL;
  if(!in_use)
    intrB = new((void *)&tb_refB_obj[0]) Cyg_Interrupt(CYGNUM_HAL_INTERRUPT_SIU_TB_B,
                                                       CYGNUM_HAL_INTERRUPT_SIU_TB_B_PRIORITY,
                                                       (CYG_ADDRWORD)222,
                                                       &isr_tb_refB,
                                                       &dsr_tb_refB);

  HAL_INTERRUPT_IN_USE(CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC, in_use);
  Cyg_Interrupt * intrRTCsec = NULL;
  if(!in_use)
    intrRTCsec = new((void *)&rtc_sec_obj[0]) Cyg_Interrupt(CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC,
                                                            CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC_PRIORITY,
                                                            (CYG_ADDRWORD)333,
                                                            &isr_rtc_sec,
                                                            &dsr_rtc_sec);

  HAL_INTERRUPT_IN_USE(CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR, in_use);
  Cyg_Interrupt * intrRTCalr = NULL;
  if(!in_use)
    intrRTCalr = new((void *)&rtc_alr_obj[0]) Cyg_Interrupt(CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR,
                                                            CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR_PRIORITY,
                                                            (CYG_ADDRWORD)444,
                                                            &isr_rtc_alr,
                                                            &dsr_rtc_alr);

  HAL_INTERRUPT_IN_USE(CYGNUM_HAL_INTERRUPT_SIU_PIT, in_use);
  Cyg_Interrupt * intrPIT = NULL;
  if(!in_use)
    intrPIT = new((void *)&pit_obj[0]) Cyg_Interrupt(CYGNUM_HAL_INTERRUPT_SIU_PIT,
                                                     CYGNUM_HAL_INTERRUPT_SIU_PIT_PRIORITY,
                                                     (CYG_ADDRWORD)555,
                                                     &isr_pit,
                                                     &dsr_pit);

  HAL_INTERRUPT_IN_USE(CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC, in_use);
  Cyg_Interrupt * intrTx = NULL;
  if(!in_use)
    intrTx = new((void *)&tx_obj[0]) Cyg_Interrupt(CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC,
                                                   CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC_PRIORITY,
                                                   (CYG_ADDRWORD)666,
                                                   &isr_tx,
                                                   &dsr_tx);

  // Set references for the timebase
  CYGARC_TBSETREFA(2500000);
  CYGARC_TBSETREFB(5000000);
  CYGARC_CLEARTB;

  // Setup a 5 seconds alarm
  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_RTCAL);
  CYGARC_RTCSETALARM(5);
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_RTCAL);
  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_RTC);
  CYGARC_RTCRESET;
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_RTC);

  // Setup PIT
  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_PITC);
  CYGARC_SETPITCOUNT(0x500);
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_PITC);

  // Setup SCI2
  cyg_uint16 val;
  val = 130;
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SCC2R0, val);

  // attach the interrupt and enable all interrupts
  if(intrA)
    intrA->attach();

  if(intrB)
    intrB->attach();

  if(intrRTCsec)
    intrRTCsec->attach();

  if(intrRTCalr)
    intrRTCalr->attach();

  if(intrPIT)
    intrPIT->attach();

  if(intrTx)
    intrTx->attach();
 
//  Cyg_Interrupt::unmask_interrupt(CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC);
  Cyg_Interrupt::unmask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_TB_A);
  Cyg_Interrupt::unmask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_TB_B);
  Cyg_Interrupt::unmask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC);
  Cyg_Interrupt::unmask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR);
  Cyg_Interrupt::unmask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_PIT);

  cyg_uint16 i = 0;
  while(seconds < 30)
    testfunc(&i);

  Cyg_Interrupt::mask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_PIT);
  Cyg_Interrupt::mask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR);
  Cyg_Interrupt::mask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC);
  Cyg_Interrupt::mask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_TB_A);
  Cyg_Interrupt::mask_interrupt(CYGNUM_HAL_INTERRUPT_SIU_TB_B);
//  Cyg_Interrupt::mask_interrupt(CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC);

  return 0;
}
