#ifndef MPC555_ISRSYS_CXX
#define MPC555_ISRSYS_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/hal/mpc555_isrsys.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/tb_isrdispatch.h"
#include "cyg/hal/rtc_isrdispatch.h"
#include "cyg/hal/pit_isrdispatch.h"
#include "cyg/hal/pll_isrdispatch.h"
#include "cyg/hal/imb3_isrdispatch.h"
#include "cyg/hal/quadc_isrsource.h"
#include "cyg/hal/qsci_isrsource.h"
#include "cyg/hal/toucan_isrsource.h"
#include "cyg/hal/mios_isrsource.h"
#include "cyg/hal/tpu_isrsource.h"

//---------------------------
// Create the ISR dispatchers
//---------------------------
TimeBase_isrDispatcher tbDispatcher CYG_INIT_PRIORITY(HAL);
RTC_isrDispatcher      rtcDispatcher CYG_INIT_PRIORITY(HAL);
PIT_isrDispatcher      pitDispatcher CYG_INIT_PRIORITY(HAL);
PLL_isrDispatcher      pllDispatcher CYG_INIT_PRIORITY(HAL);
IMB3_isrDispatcher     imb3Dispatcher0 CYG_INIT_PRIORITY(HAL);
IMB3_isrDispatcher     imb3Dispatcher1 CYG_INIT_PRIORITY(HAL);
IMB3_isrDispatcher     imb3Dispatcher2 CYG_INIT_PRIORITY(HAL);
IMB3_isrDispatcher     imb3Dispatcher3 CYG_INIT_PRIORITY(HAL);

//--------------------------------------
// And create the IMB3 interrupt sources
//--------------------------------------
TPU_isrSource    tpu_a(&imb3Dispatcher0,    ISR_SOURCE_PRIORITY_TPU_A,    TPU_isrSource::TPU_A) CYG_INIT_PRIORITY(HAL);
TPU_isrSource    tpu_b(&imb3Dispatcher3,    ISR_SOURCE_PRIORITY_TPU_B,    TPU_isrSource::TPU_B) CYG_INIT_PRIORITY(HAL);
TOUCAN_isrSource toucan_a(&imb3Dispatcher1, ISR_SOURCE_PRIORITY_TOUCAN_A, TOUCAN_isrSource::TOUCAN_A) CYG_INIT_PRIORITY(HAL);
TOUCAN_isrSource toucan_b(&imb3Dispatcher3, ISR_SOURCE_PRIORITY_TOUCAN_B, TOUCAN_isrSource::TOUCAN_B) CYG_INIT_PRIORITY(HAL);
MIOS_isrSource   mios_a(&imb3Dispatcher2,   ISR_SOURCE_PRIORITY_MIOS_A,   MIOS_isrSource::MIOS_A) CYG_INIT_PRIORITY(HAL);
MIOS_isrSource   mios_b(&imb3Dispatcher3,   ISR_SOURCE_PRIORITY_MIOS_B,   MIOS_isrSource::MIOS_B) CYG_INIT_PRIORITY(HAL);
QUADC_isrSource  quadc_a(&imb3Dispatcher3,  ISR_SOURCE_PRIORITY_QUADC_A_QUEUE1,
                                            ISR_SOURCE_PRIORITY_QUADC_A_QUEUE2,
                                            QUADC_isrSource::QUADC_A) CYG_INIT_PRIORITY(HAL);
QUADC_isrSource  quadc_b(&imb3Dispatcher3,  ISR_SOURCE_PRIORITY_QUADC_B_QUEUE1,
                                            ISR_SOURCE_PRIORITY_QUADC_B_QUEUE2,
                                            QUADC_isrSource::QUADC_B) CYG_INIT_PRIORITY(HAL);
QSCI_isrSource   qsci(&imb3Dispatcher3,     ISR_SOURCE_PRIORITY_QSCI,
                                            ISR_SOURCE_PRIORITY_QSPI) CYG_INIT_PRIORITY(HAL);

//--------------------------
// The prestart startup code
//--------------------------
externC void
cyg_prestart(void)
{
  if(tbDispatcher.attach(ISR_DISPATCHER_LEVEL_TB))
  {
    tbDispatcher.enableIsrDispatching();
    tbDispatcher.enableInterrupts();
  }
  
  if(pitDispatcher.attach(ISR_DISPATCHER_LEVEL_PIT))
  {
    pitDispatcher.enableIsrDispatching();
    pitDispatcher.enableInterrupts();
  }

  if(pllDispatcher.attach(ISR_DISPATCHER_LEVEL_PLL))
  {
    pllDispatcher.enableIsrDispatching();
    pllDispatcher.enableInterrupts();
  }

  if(rtcDispatcher.attach(ISR_DISPATCHER_LEVEL_RTC))
  {
    rtcDispatcher.enableIsrDispatching();
    rtcDispatcher.enableInterrupts();
  }

  if(imb3Dispatcher0.attach(ISR_DISPATCHER_LEVEL_IMB3_0))
  {
    imb3Dispatcher0.enableIsrDispatching();
    imb3Dispatcher0.enableInterrupts();
  }

  if(imb3Dispatcher1.attach(ISR_DISPATCHER_LEVEL_IMB3_1))
  {
    imb3Dispatcher1.enableIsrDispatching();
    imb3Dispatcher1.enableInterrupts();
  }

  if(imb3Dispatcher2.attach(ISR_DISPATCHER_LEVEL_IMB3_2))
  {
    imb3Dispatcher2.enableIsrDispatching();
    imb3Dispatcher2.enableInterrupts();
  }

  if(imb3Dispatcher3.attach(ISR_DISPATCHER_LEVEL_IMB3_3))
  {
    imb3Dispatcher3.enableIsrDispatching();
    imb3Dispatcher3.enableInterrupts();
  }
}
#endif // if !defined ...

#endif // MPC555_ISRSYS_CXX
