#ifndef RTC_ISRDISPATCH_CXX
#define RTC_ISRDISPATCH_CXX

//----------------------------------
// includes and forward declarations
//----------------------------------
#include "cyg/hal/rtc_isrdispatch.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/hal_io.h"
#include "cyg/hal/hal_intr.h"

//---------------------
// Class implementation
//---------------------
// Public constructors(s) and destructor
RTC_isrDispatcher::RTC_isrDispatcher()
    : HAL_isrDispatcher()
{
}

// Provide an (empty) virtual destructor
RTC_isrDispatcher::~RTC_isrDispatcher()
{
}

//------------------------------------------
// Protected member functions implementation
//------------------------------------------
// findInterruptingVector
cyg_uint32 RTC_isrDispatcher::findInterruptingVector(void)
{
  cyg_uint16 rtcsc;

  HAL_READ_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);
  if((rtcsc & CYGARC_REG_IMM_RTCSC_SEC) && (rtcsc & CYGARC_REG_IMM_RTCSC_SIE))
    return CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC;

  if((rtcsc & CYGARC_REG_IMM_RTCSC_ALR) && (rtcsc & CYGARC_REG_IMM_RTCSC_ALE))
    return CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR;

  return 0;
}

// setInterruptingLevel
bool RTC_isrDispatcher::setInterruptingLevel(unsigned char a_level)
{
  cyg_uint16 rtcsc;
  cyg_uint16 level = (CYGARC_REG_IMM_RTCSC_IRQ0 >> a_level);

  HAL_READ_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);
  rtcsc &= ~((cyg_uint16)CYGARC_REG_IMM_RTCSC_IRQMASK);
  rtcsc |= level;
  rtcsc &= ~(CYGARC_REG_IMM_RTCSC_SEC); // Prevent from clearing interrupt flags
  rtcsc &= ~(CYGARC_REG_IMM_RTCSC_ALR); // accidently. Just do what is asked.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);

  return true;
}

// enableInterruptingDevice
void RTC_isrDispatcher::enableInterruptingDevice(void)
{
  cyg_uint16 rtcsc;

  HAL_READ_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);
  rtcsc |= CYGARC_REG_IMM_RTCSC_RTE;
  rtcsc &= ~(CYGARC_REG_IMM_RTCSC_SEC); // Prevent from clearing interrupt flags
  rtcsc &= ~(CYGARC_REG_IMM_RTCSC_ALR); // accidently. Just do what is asked.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);
}

// disableInterruptingDevice
void RTC_isrDispatcher::disableInterruptingDevice(void)
{
  cyg_uint16 rtcsc;

  HAL_READ_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);
  rtcsc &= ~((cyg_uint16)CYGARC_REG_IMM_RTCSC_RTE);
  rtcsc &= ~(CYGARC_REG_IMM_RTCSC_SEC); // Prevent from clearing interrupt flags
  rtcsc &= ~(CYGARC_REG_IMM_RTCSC_ALR); // accidently. Just do what is asked.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_RTCSC, rtcsc);
}
#endif // if !defined

#endif // RTC_ISRDISPATCH_CXX
