#ifndef CMS_OBJECT_H
#define CMS_OBJECT_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/io/comobj.h"
#include "cyg/io/cmserr.h"
#include <cyg/kal/ksync.h>

//-------------------
// Class declarations
//-------------------
// class CMS_Object
class CMS_Object : public CommunicationObjectClient
{
  public:
    typedef enum {
      Client, Server } CMS_UserType;

    CMS_UserType           getUserType(void)
                           {
                           return m_usertype;
                           }
    unsigned int           getInhibittime(void)
                           {
                           return m_inhibittime;
                           }
    unsigned char          getPriority(void)
                           {
                           return m_priority;
                           }
    const char *           getName(void)
                           {
                           return m_name;
                           }
    void                   define(void);

    // Overloaded virtuals from base class CommunicationObjectClient
    virtual void           dataIndication(unsigned char a_data[8],
                                          unsigned char a_length,
                                          CommunicationObject * a_cob) = 0;

    // Give the application a chance to encode an decode application specific error
    // responses
    void                   encode(void * a_dst, const void * a_src, size_t a_size);
    void                   decode(void * a_dst, const void * a_src, size_t a_size);

  protected:
    // Prevent from instantiating the class directly
                           CMS_Object(unsigned int a_inhibittime,
                                      unsigned char a_priority,
                                      const char * a_name,
                                      bool a_numeric = true);
    virtual                ~CMS_Object();

    CMS_UserType           m_usertype;
    unsigned int           m_inhibittime;
    unsigned char          m_priority;
    char                   m_name[14];
    CommunicationObject *  m_txcob;
    CommunicationObject *  m_rxcob;
    bool                   m_numeric;
};

class CMS_ServerObject : public CMS_Object
{
  protected:
                           CMS_ServerObject(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            bool a_numeric = true);
    virtual                ~CMS_ServerObject();

  public:
    virtual void           dataIndication(unsigned char * a_data,
                                          unsigned char a_length,
                                          CommunicationObject * a_cob) = 0;
};

class CMS_ClientObject : public CMS_Object
{
  protected:
                           CMS_ClientObject(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            bool a_numeric = true);
    virtual                ~CMS_ClientObject();

  public:
    virtual void           dataIndication(unsigned char * a_data,
                                          unsigned char a_length,
                                          CommunicationObject * a_cob) = 0;
};

#endif // CMS_OBJECT_H
