#ifndef CMSOBJ_CXX
#define CMSOBJ_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/io/cmsobj.h"
#include "pkgconf/io_cal.h"
#include <string.h>

//----------------------
// Class implementations
//----------------------
// Constructor
CMS_Object::CMS_Object(unsigned int a_inhibittime,
                       unsigned char a_priority,
                       const char * a_name,
                       bool a_numeric)
    : m_inhibittime(a_inhibittime),
      m_priority(a_priority),
      m_txcob(0),
      m_rxcob(0),
      m_numeric(a_numeric)
{ // CMS name may not be longer than 13 characters
  strncpy(m_name, a_name, 14);
  m_name[13] = 0; // Make sure it is NULL terminated
}

// Privide an (empty) virtual destructor
CMS_Object::~CMS_Object()
{
  if(m_txcob)
    m_txcob->destroy();
  if(m_rxcob)
    m_rxcob->destroy();

  delete m_txcob;
  delete m_rxcob;
}

//---------------------
// Other public members
//---------------------
// define
void CMS_Object::define(void)
{
  if(m_txcob)
    m_txcob->create();
  if(m_rxcob)
    m_rxcob->create();
}

// encode
void CMS_Object::encode(void * a_dst, const void * a_src, size_t a_size)
{
#if defined(BIG_ENDIAN_TARGET)
  if(m_numeric)
  { // Only numeric values have to encoded little endian
    unsigned char * p = (unsigned char *)a_dst;
    unsigned char * q = (unsigned char *)a_src + a_size - 1;

    while(a_size)
    {
      *p = *q;
      p++;
      q--;
      a_size--;
    }
  }
  else
#endif
    memcpy(a_dst, a_src, a_size);
}

// decode
void CMS_Object::decode(void * a_dst, const void * a_src, size_t a_size)
{
#if defined(BIG_ENDIAN_TARGET)
  if(m_numeric)
  { // Only numeric values have to be encoded little endian
    unsigned char * p = (unsigned char *)a_dst;
    unsigned char * q = (unsigned char *)a_src + a_size - 1;

    while(a_size)
    {
      *p = *q;
      p++;
      q--;
      a_size--;
    }
  }
  else
#endif
    memcpy(a_dst, a_src, a_size);
}

// Constructor
CMS_ServerObject::CMS_ServerObject(unsigned int a_inhibittime,
                                   unsigned char a_priority,
                                   const char * a_name,
                                   bool a_numeric)
    : CMS_Object(a_inhibittime, a_priority, a_name, a_numeric)
{
  m_usertype = Server;
}

// Provide an (empty) virtual destructor
CMS_ServerObject::~CMS_ServerObject()
{
}

// Constructor
CMS_ClientObject::CMS_ClientObject(unsigned int a_inhibittime,
                                   unsigned char a_priority,
                                   const char * a_name,
                                   bool a_numeric)
    : CMS_Object(a_inhibittime, a_priority, a_name, a_numeric)
{
  m_usertype = Client;
}

// Provide an (empty) virtual destructor
CMS_ClientObject::~CMS_ClientObject()
{
}

#endif // CMSOBJ_CXX
