#ifndef CAL_H
#define CAL_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/io/calio.h"
#include <cyg/io/devtab.h>
#include <cyg/error/codes.h>
#include <pkgconf/io_cal.h>
#include <cyg/io/canmsgdisp.h>

//--------------------
// Function prototypes
//--------------------
static bool cal_init(struct cyg_devtab_entry * a_tab);
static Cyg_ErrNo cal_lookup(struct cyg_devtab_entry ** a_tab,
                            struct cyg_devtab_entry * a_sub_tab,
                            const char * a_name);
static Cyg_ErrNo cal_set_config(cyg_io_handle_t a_handle, cyg_uint32 a_key, const void * a_buf, cyg_uint32 * a_len);
static Cyg_ErrNo cal_get_config(cyg_io_handle_t a_handle, cyg_uint32 a_key, void * a_buf, cyg_uint32 * a_len);

//-----------------
// Type definitions
//-----------------
struct cal_private_info {
  cyg_io_handle_t        m_dev_handle;
  CanMessageDispatcher * m_dispatcher;
};

#define INIT_CAL_PRIVATE_INFO(_info, _dev_handle, _dispatcher) \
  struct cal_private_info _info = {                            \
    _dev_handle,                                               \
    _dispatcher                                                \
}

//------------------------------------
// Register the driver with the kernel
//------------------------------------
static DEVIO_TABLE(cal_devio,
                   0,   // No write function, cal is provided as a C++ library
                   0,   // No read function, cal is provided as a C++ library
                   cal_get_config,
                   cal_set_config);

#ifdef CYGPKG_IO_CAL_CAL0
static CanMessageDispatcher cal0_dispatcher("CAL0_dispatcher");
INIT_CAL_PRIVATE_INFO(cal_private_info0, 0, &cal0_dispatcher);
DEVTAB_ENTRY(cal_io0,
             CYGDAT_IO_CAL_MPC555_CAL_A_NAME,
             CYGDAT_IO_CAL_CALA_DEV,
             &cal_devio,
             cal_init,
             cal_lookup,
             &cal_private_info0);
#endif

#ifdef CYGPKG_IO_CAL_CAL1
static CanMessageDispatcher cal1_dispatcher("CAL1_dispatcher");
INIT_CAL_PRIVATE_INFO(cal_private_info1, 0, &cal1_dispatcher);
DEVTAB_ENTRY(cal_io1,
             CYGDAT_IO_CAL_MPC555_CAL_B_NAME,
             CYGDAT_IO_CAL_CALB_DEV,
             &cal_devio,
             cal_init,
             cal_lookup,
             &cal_private_info1);
#endif

//-------------------------
// Functions implementation
//-------------------------
// cal_init
static bool cal_init(struct cyg_devtab_entry * a_tab)
{
  return true;
}

// cal_lookup
static Cyg_ErrNo cal_lookup(struct cyg_devtab_entry ** a_tab,
                            struct cyg_devtab_entry * a_sub_tab,
                            const char * a_name)
{
  cyg_io_handle_t chan = (cyg_io_handle_t)a_sub_tab;
  struct cal_private_info * priv = (struct cal_private_info *)(*a_tab)->priv;
  priv->m_dev_handle = chan;

  priv->m_dispatcher->start(priv->m_dev_handle);

  return ENOERR;
}

// cal_set_config
static Cyg_ErrNo cal_set_config(cyg_io_handle_t a_handle, cyg_uint32 a_key, const void * a_buf, cyg_uint32 * a_len)
{
  cyg_devtab_entry_t * t = (cyg_devtab_entry_t *)a_handle;
  struct cal_private_info * priv = (struct cal_private_info *)t->priv;
  cyg_io_handle_t chan = (cyg_io_handle_t)priv->m_dev_handle;
  Cyg_ErrNo result = ENOERR;

  switch(a_key)
  {
    default: // Pass on to the lower level driver
      result = cyg_io_set_config(chan, a_key, a_buf, a_len);
      break;
  }

  return result;
}

// cal_get_config
static Cyg_ErrNo cal_get_config(cyg_io_handle_t a_handle, cyg_uint32 a_key, void * a_buf, cyg_uint32 * a_len)
{
  cyg_devtab_entry_t * t = (cyg_devtab_entry_t *)a_handle;
  struct cal_private_info * priv = (struct cal_private_info *)t->priv;
  cyg_io_handle_t chan = (cyg_io_handle_t)priv->m_dev_handle;
  Cyg_ErrNo result = ENOERR;

  switch(a_key)
  {
    case CAL_IO_GET_CONFIG_MSGDISPATCHER:
      {
      if(*a_len != sizeof(CanMessageDispatcher *))
        return -EINVAL;

      *(CanMessageDispatcher **)a_buf = priv->m_dispatcher;
      }
      break;
    case CAL_IO_GET_CONFIG_CANHANDLE:
      {
      if(*a_len != sizeof(cyg_io_handle_t))
        return -EINVAL;

      *(cyg_io_handle_t *)a_buf = priv->m_dev_handle;
      }
      break;
    default: // Pass on to the lower level driver
      result = cyg_io_get_config(chan, a_key, a_buf, a_len);
      break;
  }

  return result;
}

#endif // CAL_H
