#ifndef MPC555_CAN_WITH_INTS_C
#define MPC555_CAN_WITH_INTS_C

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <pkgconf/io_can.h>

// Only build this driver for the MPC555 based CME555 board
#ifdef CYGPKG_IO_CAN_MPC555_CME555
#include <cyg/hal/drv_api.h>
#include <cyg/io/canio.h>
#include <cyg/io/can.h>
#include <cyg/io/devtab.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/error/codes.h>
#include "pkgconf/io_can.h"   // include this file in the dependencies
#include "mpc555_can.h"

//-----------------
// Type definitions
//-----------------
typedef struct mpc555_can_info {
  CYG_ADDRWORD   m_base;   // The base address of the port

  CYG_WORD       m_mb0_interrupt_num;
  CYG_WORD       m_mb1_interrupt_num;
  CYG_WORD       mb15_interrupt_num;
  cyg_priority_t m_mb0_interrupt_priority;  // The first receiver interrupt
  cyg_priority_t m_mb1_interrupt_priority;  // The second receiver interrupt
  cyg_priority_t mb15_interrupt_priority;   // The transmitter interrupt

  CYG_WORD       m_boff_interrupt_num;
  cyg_priority_t m_boff_interrupt_priority;

  cyg_uint8      m_imb3_interrupt_priority;

  cyg_interrupt  m_mb0_interrupt;
  cyg_interrupt  m_mb1_interrupt;
  cyg_interrupt  mb15_interrupt;
  cyg_handle_t   m_mb0_interrupt_handle;
  cyg_handle_t   m_mb1_interrupt_handle;
  cyg_handle_t   mb15_interrupt_handle;

  cyg_interrupt  m_boff_interrupt;
  cyg_handle_t   m_boff_interrupt_handle;
} mpc555_can_info;

// Note
// Making the lowest two buffers the receivers and the highest the transmitter
// Allows that even remote frames will be received by the receive buffers. This
// Way, we do not need to make the transmitter buffer a "temporary" receiver for receiving
// remote frames. Buffer 15 is allways the transmitter, regardless of the fact if the
// previously transmitted frame was a remote request or not.

//--------------------
// Function prototypes
//--------------------
static bool mpc555_can_init(struct cyg_devtab_entry * a_tab);
static Cyg_ErrNo mpc555_can_lookup(struct cyg_devtab_entry ** a_tab,
                                   struct cyg_devtab_entry * a_sub_tab,
                                   const char * a_name);
static bool mpc555_can_set_config(can_channel * a_chan, can_info_t * a_config);
static bool mpc555_can_xmt_message(can_channel * a_chan, const can_message * a_msg);
static void mpc555_can_start_xmit(can_channel * a_chan);
static bool mpc555_define_rx_action1(can_channel * a_chan, can_rxid_action_t * a_action);
static bool mpc555_define_rx_action2(can_channel * a_chan, can_rxid_action_t * a_action);
static bool mpc555_add_rx_id(can_channel * a_chan, cyg_uint16 a_id);
static bool mpc555_remove_rx_id(can_channel * a_chan, cyg_uint16 a_id);
static bool mpc555_add_rtr_id(can_channel * a_chan, cyg_uint16 a_id);
static bool mpc555_remove_rtr_id(can_channel * a_chan, cyg_uint16 a_id);
static bool mpc555_update_rtr_buffer(can_channel * a_chan, const can_message * a_msg);
static bool mpc555_can_reset(can_channel * a_chan);

static cyg_uint32 mpc555_mb0_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data);
static cyg_uint32 mpc555_mb1_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data);
static cyg_uint32 mpc555_mb15_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void mpc555_mb0_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data);
static void mpc555_mb1_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data);
static void mpc555_mb15_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);
static cyg_uint32 mpc555_boff_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data);
static void mpc555_boff_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data);

//----------------------------
// Private function prototypes
//----------------------------
static void toucan_MB_off(cyg_addrword_t a_port);
static void toucan_MB_init(cyg_addrword_t a_port);
static void toucan_bus_on(cyg_addrword_t a_port);
static void toucan_RxMessage(t_can_messagebuffer * a_messagebuffer, can_channel * a_channel);
static void toucan_TxMessage(t_can_messagebuffer * a_messagebuffer, const can_message * a_msg, can_channel * a_channel);

//------------------------------------
// Register the driver with the kernel
//------------------------------------
static CAN_FUNS(mpc555_can_funs,
                mpc555_can_set_config,
                mpc555_can_xmt_message,
                mpc555_can_start_xmit,
                mpc555_define_rx_action1,
                mpc555_define_rx_action2,
                mpc555_add_rx_id,
                mpc555_remove_rx_id,
                mpc555_add_rtr_id,
                mpc555_remove_rtr_id,
                mpc555_update_rtr_buffer,
                mpc555_can_reset);

#ifdef CYGPKG_IO_CAN_MPC555_CME555_CAN_A
static mpc555_can_info mpc555_can_info0 = {TOUCAN_A_BASE,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B0,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B1,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B15,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B0_PRIORITY,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B1_PRIORITY,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B15_PRIORITY,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_BOFF,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_BOFF_PRIORITY,
                                           ISR_SOURCE_PRIORITY_TOUCAN_A};

can_message toucan_A_output_buffer[CYGNUM_IO_CAN_MPC555_CME555_CAN_A_OUTBUFSIZE];
can_message toucan_A_input_buffer[CYGNUM_IO_CAN_MPC555_CME555_CAN_A_INBUFSIZE];

CAN_CHANNEL(mpc555_can_channel0,
            mpc555_can_funs,
            mpc555_can_info0,
            CAN_BAUD_RATE(CYGNUM_IO_CAN_MPC555_CME555_CAN_A_BAUD),
            CAN_RX_MODE_DEFAULT,
            CAN_TX_MODE_DEFAULT,
            toucan_A_output_buffer, CYGNUM_IO_CAN_MPC555_CME555_CAN_A_OUTBUFSIZE,
            toucan_A_input_buffer, CYGNUM_IO_CAN_MPC555_CME555_CAN_A_INBUFSIZE);

DEVTAB_ENTRY(mpc555_can_io0,
             CYGDAT_IO_CAN_MPC555_CME555_CAN_A_NAME,
             0, // Does not depend on any lower level device drivers
             &can_devio,
             mpc555_can_init,
             mpc555_can_lookup,
             &mpc555_can_channel0);
#endif // ifdef CYGPKG_IO_CAN_MPC555_CAN_A

#ifdef CYGPKG_IO_CAN_MPC555_CME555_CAN_B
static mpc555_can_info mpc555_can_info1 = {TOUCAN_B_BASE,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B0,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B1,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B15,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B0_PRIORITY,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B1_PRIORITY,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B15_PRIORITY,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_BOFF,
                                           CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_BOFF_PRIORITY,
                                           ISR_SOURCE_PRIORITY_TOUCAN_B};

can_message toucan_B_output_buffer[CYGNUM_IO_CAN_MPC555_CME555_CAN_B_OUTBUFSIZE];
can_message toucan_B_input_buffer[CYGNUM_IO_CAN_MPC555_CME555_CAN_B_INBUFSIZE];

CAN_CHANNEL(mpc555_can_channel1,
            mpc555_can_funs,
            mpc555_can_info1,
            CAN_BAUD_RATE(CYGNUM_IO_CAN_MPC555_CME555_CAN_B_BAUD),
            CAN_RX_MODE_DEFAULT,
            CAN_TX_MODE_DEFAULT,
            toucan_B_output_buffer, CYGNUM_IO_CAN_MPC555_CME555_CAN_B_OUTBUFSIZE,
            toucan_B_input_buffer, CYGNUM_IO_CAN_MPC555_CME555_CAN_B_INBUFSIZE);

DEVTAB_ENTRY(mpc555_can_io1,
             CYGDAT_IO_CAN_MPC555_CME555_CAN_B_NAME,
             0, // Does not depend on any lower level device drivers
             &can_devio,
             mpc555_can_init,
             mpc555_can_lookup,
             &mpc555_can_channel1);
#endif // ifdef CYGPKG_IO_CAN_MPC555_CAN_B

//-------------------------
// Function implementations
//-------------------------
static bool mpc555_can_config_port(can_channel * a_chan, can_info_t * a_new_config, bool a_init)
{
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)(a_chan->m_dev_priv);
  cyg_addrword_t port = mpc555_chan->m_base;
  cyg_uint16 canctrl0_canctrl1;
  cyg_uint16 tcnmcr;
  cyg_uint16 old_errmask;
  cyg_uint16 old_boffmask;
  cyg_uint16 old_wakemask;
  cyg_uint16 presdiv_ctrl2;

  unsigned char * tx_mode = 0;
  unsigned char * rx_mode = 0;

  cyg_uint16 irl  = mpc555_chan->m_imb3_interrupt_priority % 8;
  cyg_uint16 ilbs = mpc555_chan->m_imb3_interrupt_priority / 8;
  cyg_uint16 canicr;
 
  // Validate the configuration
  t_can_bittiming * bittiming = 0;
  if((a_new_config->m_baud >= CAN_BAUD_KBIT_1000) && 
     (a_new_config->m_baud <= CAN_BAUD_KBIT_10))  
    bittiming = &CiA_bittiming[a_new_config->m_baud];
  else
    return false; // Unsupported baudrate

  if((a_new_config->m_rx_mode >= CAN_RX_LOGICAL_ZERO_DOMINANT) && 
     (a_new_config->m_rx_mode <= CAN_RX_LOGICAL_ONE_DOMINANT))
    rx_mode = &RxMode[a_new_config->m_rx_mode];
  else
    return false;

  if((a_new_config->m_tx_mode >= CAN_TX_FULL_CMOS_POSITIVE_POLARITY) && 
     (a_new_config->m_tx_mode <= CAN_TX_OPEN_DRAIN))
    tx_mode = &TxMode[a_new_config->m_tx_mode];
  else
    return false;

  // Disable interrupts while changing the configuration. Store the old interrupt state
  // so we can restore it if necessary
  HAL_READ_UINT16(port + CANCTRL0_CANCTRL1, canctrl0_canctrl1);
  HAL_READ_UINT16(port + TCNMCR, tcnmcr);
  old_wakemask = tcnmcr;
  old_errmask   = canctrl0_canctrl1;
  old_boffmask  = canctrl0_canctrl1;
  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_BOFFMSK);
  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_ERRMSK);
  tcnmcr            &= ~((cyg_uint16)TCNMCR_WAKEMSK);
  HAL_WRITE_UINT16(port + CANCTRL0_CANCTRL1, canctrl0_canctrl1);
  HAL_WRITE_UINT16(port + TCNMCR, tcnmcr);
  old_wakemask &= TCNMCR_WAKEMSK;
  old_errmask  &= CANCTRL0_CANCTRL1_ERRMSK;
  old_boffmask &= CANCTRL0_CANCTRL1_BOFFMSK;

  // Ok, now we can program the interrupt level in the device
  HAL_READ_UINT16(port + CANICR, canicr);
  irl  = irl  << CANICR_IRL_SHIFT;
  ilbs = ilbs << CANICR_ILBS_SHIFT;
  irl  &= (cyg_uint16)CANICR_IRL;
  ilbs &= (cyg_uint16)CANICR_ILBS;
  canicr &= ~((cyg_uint16)CANICR_IRL);
  canicr &= ~((cyg_uint16)CANICR_ILBS);
  canicr |= irl;
  canicr |= ilbs;
  HAL_WRITE_UINT16(port + CANICR, canicr);

  // Ok, so far so good, program the desired bittimings in the device
  HAL_READ_UINT16(port + CANCTRL0_CANCTRL1, canctrl0_canctrl1);
  HAL_READ_UINT16(port + PRESDIV_CTRL2, presdiv_ctrl2);

  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_SAMP);
  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_PROPSE);
  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_LBUF);     // Highest priority buffer first
  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_RXMOD);
  canctrl0_canctrl1 &= ~((cyg_uint16)CANCTRL0_CANCTRL1_TXMOD);

  if(bittiming->m_highsample)
    canctrl0_canctrl1 |= CANCTRL0_CANCTRL1_SAMP;
  canctrl0_canctrl1 |= (cyg_uint16)(bittiming->m_propseg);        // Program the propseg
  canctrl0_canctrl1 |= ((cyg_uint16)(*rx_mode)) << CANCTRL0_CANCTRL1_RXMOD_SHIFT;
  canctrl0_canctrl1 |= ((cyg_uint16)(*tx_mode)) << CANCTRL0_CANCTRL1_TXMOD_SHIFT;
  HAL_WRITE_UINT16(port + CANCTRL0_CANCTRL1, canctrl0_canctrl1);

  presdiv_ctrl2 &= ~((cyg_uint16)PRESDIV_CTRL2_PRESDIV);
  presdiv_ctrl2 |= ((cyg_uint16)(bittiming->m_prescaler)) << PRESDIV_CTRL2_PRESDIV_SHIFT;

  presdiv_ctrl2 &= ~((cyg_uint16)PRESDIV_CTRL2_RJW);
  presdiv_ctrl2 &= ~((cyg_uint16)PRESDIV_CTRL2_PSEG1);
  presdiv_ctrl2 &= ~((cyg_uint16)PRESDIV_CTRL2_PSEG2);

  presdiv_ctrl2 |= ((cyg_uint16)(bittiming->m_resync)) << PRESDIV_CTRL2_RJW_SHIFT;
  presdiv_ctrl2 |= ((cyg_uint16)(bittiming->m_pseg1)) << PRESDIV_CTRL2_PSEG1_SHIFT;
  presdiv_ctrl2 |= ((cyg_uint16)(bittiming->m_pseg2));
  HAL_WRITE_UINT16(port + PRESDIV_CTRL2, presdiv_ctrl2);

  // And finally, restore or enable interrupts
  HAL_READ_UINT16(port + CANCTRL0_CANCTRL1, canctrl0_canctrl1);
  HAL_READ_UINT16(port + TCNMCR, tcnmcr);

  tcnmcr |= TCNMCR_FRZ;
  tcnmcr |= TCNMCR_HALT;
  tcnmcr |= TCNMCR_SELFWAKE;
  tcnmcr |= TCNMCR_APS;  

  if(a_init)
  { // Enable Boff interrupt
    canctrl0_canctrl1 |= CANCTRL0_CANCTRL1_BOFFMSK;
  }
  else
  { // Restore the old interrupt state
    canctrl0_canctrl1 |= old_errmask;
    canctrl0_canctrl1 |= old_boffmask;
    tcnmcr            |= old_wakemask;
  }
  HAL_WRITE_UINT16(port + CANCTRL0_CANCTRL1, canctrl0_canctrl1);
  HAL_WRITE_UINT16(port + TCNMCR, tcnmcr);

  // Save the configuration in the channel
  if(a_new_config != &a_chan->m_config)
    a_chan->m_config = *a_new_config;

  return true;
}

// mpc555_can_reset
// Reset the controller, this is identical to the initialization, except for the creation of the interrupts
static bool mpc555_can_reset(can_channel * a_chan)
{
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  // Soft reset the CAN 2.0B controller
  cyg_uint16 tcnmcr = TCNMCR_FRZ | TCNMCR_HALT | TCNMCR_SOFTRST;
  HAL_WRITE_UINT16(port + TCNMCR, tcnmcr);
  asm("nop"); // Wait one cycle before accessing MCR again
  while(tcnmcr & TCNMCR_SOFTRST)
    HAL_READ_UINT16(port + TCNMCR, tcnmcr);

  if(!mpc555_can_config_port(a_chan, &a_chan->m_config, true))
    return false;

  a_chan->m_callbacks->can_init(a_chan, true);

  toucan_MB_off(port); // Set all 16 Toucan MessageBuffers to inactive

  toucan_MB_init(port); // initialize the toucan message buffers
  toucan_bus_on(port);  // allow the TOUCAN CAN 2.0B CAN controller to take part in the bus arbitration

  // Enable the interrupts
  cyg_drv_interrupt_unmask(mpc555_chan->m_mb0_interrupt_num);
  cyg_drv_interrupt_unmask(mpc555_chan->m_mb1_interrupt_num);
  cyg_drv_interrupt_unmask(mpc555_chan->mb15_interrupt_num);

  // Enable all other TOUCAN CAN 2.0B CAN Controller interrupts
  cyg_drv_interrupt_unmask(mpc555_chan->m_boff_interrupt_num);
  
  return true;
}

// mpc555_can_init
// Initialize the device. Called at bootstrap time
static bool mpc555_can_init(struct cyg_devtab_entry * a_tab)
{
  can_channel * chan = (can_channel *)a_tab->priv;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  // Soft reset the CAN 2.0B controller
  cyg_uint16 tcnmcr = TCNMCR_FRZ | TCNMCR_HALT | TCNMCR_SOFTRST;
  HAL_WRITE_UINT16(port + TCNMCR, tcnmcr);
  asm("nop"); // Wait one cycle before accessing MCR again
  while(tcnmcr & TCNMCR_SOFTRST)
    HAL_READ_UINT16(port + TCNMCR, tcnmcr);

  if(!mpc555_can_config_port(chan, &chan->m_config, true))
    return false;

  chan->m_callbacks->can_init(chan, false);

  toucan_MB_off(port); // Set all 16 Toucan MessageBuffers to inactive
  
  // Create the interrupts
  cyg_drv_interrupt_create(mpc555_chan->m_mb0_interrupt_num,
                           mpc555_chan->m_mb0_interrupt_priority,
                           (cyg_addrword_t)chan,
                           mpc555_mb0_ISR,
                           mpc555_mb0_DSR,
                           &mpc555_chan->m_mb0_interrupt_handle,
                           &mpc555_chan->m_mb0_interrupt);

  cyg_drv_interrupt_create(mpc555_chan->m_mb1_interrupt_num,
                           mpc555_chan->m_mb1_interrupt_priority,
                           (cyg_addrword_t)chan,
                           mpc555_mb1_ISR,
                           mpc555_mb1_DSR,
                           &mpc555_chan->m_mb1_interrupt_handle,
                           &mpc555_chan->m_mb1_interrupt);

  cyg_drv_interrupt_create(mpc555_chan->mb15_interrupt_num,
                           mpc555_chan->mb15_interrupt_priority,
                           (cyg_addrword_t)chan,
                           mpc555_mb15_ISR,
                           mpc555_mb15_DSR,
                           &mpc555_chan->mb15_interrupt_handle,
                           &mpc555_chan->mb15_interrupt);

  cyg_drv_interrupt_create(mpc555_chan->m_boff_interrupt_num,
                           mpc555_chan->m_boff_interrupt_priority,
                           (cyg_addrword_t)chan,
                           mpc555_boff_ISR,
                           mpc555_boff_DSR,
                           &mpc555_chan->m_boff_interrupt_handle,
                           &mpc555_chan->m_boff_interrupt);

  cyg_drv_interrupt_attach(mpc555_chan->m_mb0_interrupt_handle);
  cyg_drv_interrupt_attach(mpc555_chan->m_mb1_interrupt_handle);
  cyg_drv_interrupt_attach(mpc555_chan->mb15_interrupt_handle);
  cyg_drv_interrupt_attach(mpc555_chan->m_boff_interrupt_handle);

  return true;
}

// mpc555_can_lookup
// Called when the device is "looked up", i.e. attached
static Cyg_ErrNo mpc555_can_lookup(struct cyg_devtab_entry ** a_tab,
                                   struct cyg_devtab_entry * a_sub_tab,
                                   const char * a_name)
{
  can_channel * chan = (can_channel *)(*a_tab)->priv;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  toucan_MB_init(port); // initialize the toucan message buffers
  toucan_bus_on(port);  // allow the TOUCAN CAN 2.0B CAN controller to take part in the bus arbitration

  // Enable the interrupts
  cyg_drv_interrupt_unmask(mpc555_chan->m_mb0_interrupt_num);
  cyg_drv_interrupt_unmask(mpc555_chan->m_mb1_interrupt_num);
  cyg_drv_interrupt_unmask(mpc555_chan->mb15_interrupt_num);

  // Enable all other TOUCAN CAN 2.0B CAN Controller interrupts
  cyg_drv_interrupt_unmask(mpc555_chan->m_boff_interrupt_num);

  return ENOERR;
}

// Change the device configuration on user request
static bool mpc555_can_set_config(can_channel * a_chan, can_info_t * a_config)
{
  cyg_uint16 tcnmcr;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  // Remove the CAN controller from the bus arbitration process
  HAL_READ_UINT16(port + TCNMCR, tcnmcr);
  tcnmcr |= TCNMCR_HALT;
  HAL_WRITE_UINT16(port + TCNMCR, tcnmcr);
  
  if(mpc555_can_config_port(a_chan, a_config, false))
  {
    // Reallow the CAN controller to take part in the bus arbitration process again
    HAL_READ_UINT16(port + TCNMCR, tcnmcr);
    tcnmcr &= ~((cyg_uint16)TCNMCR_HALT);
    HAL_WRITE_UINT16(port + TCNMCR, tcnmcr);

    return true;
  }
  else
    return false;
}

// mpc555_can_xmt_message
static bool mpc555_can_xmt_message(can_channel * a_chan, const can_message * a_msg)
{
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF15);

  if(buffer->m_code == TOUCAN_MB_TX_ENABLE)
    return false; // Buffer is still transmitting data, retry later

  buffer->m_code = TOUCAN_MB_TX_DISABLE;
  toucan_TxMessage(buffer, a_msg, a_chan);

  return true;
}

// mpc555_can_start_xmt
static void mpc555_can_start_xmit(can_channel * a_chan)
{
  if(!a_chan->m_transmitting)
    (a_chan->m_callbacks->can_xmt_message)(a_chan);
}

// mpc555_define_rx_action1
static bool mpc555_define_rx_action1(can_channel * a_chan, can_rxid_action_t * a_action)
{ // Validate the identifier
  if((a_action->m_can_id & 0x7FF) != a_action->m_can_id)
    return false;

  if(!a_action->m_action)
    return false;

  (a_chan->m_action1).m_can_id   = a_action->m_can_id;
  (a_chan->m_action1).m_action = a_action->m_action;

  return true;
}

// mpc555_define_rx_action2
static bool mpc555_define_rx_action2(can_channel * a_chan, can_rxid_action_t * a_action)
{ // Validate the identifier
  if((a_action->m_can_id & 0x7FF) != a_action->m_can_id)
    return false;

  if(!a_action->m_action)
    return false;

  (a_chan->m_action2).m_can_id   = a_action->m_can_id;
  (a_chan->m_action2).m_action = a_action->m_action;

  return true;
}

// mpc555_add_rx_id
static bool mpc555_add_rx_id(can_channel * a_chan, cyg_uint16 a_id)
{
  cyg_uint8 flag;
  cyg_uint8 index;

  // Validate the parameter as an eleven bit identifier
  if((a_id & 0x7FF) != a_id)
    return false;
  
  // Use first eight bits as index, use the last three bits to set a flag on the index
  flag = (cyg_uint8)(1 << (cyg_uint8)(a_id & 0x7));
  index = (cyg_uint8)((a_id & 0x7F8) >> 3);

  a_chan->m_id_filter[index] |= flag;

  return true;
}

// mpc555_remove_rx_id
static bool mpc555_remove_rx_id(can_channel * a_chan, cyg_uint16 a_id)
{
  cyg_uint8 flag;
  cyg_uint8 index;

  // Validate the parameter as an eleven bit identifier
  if((a_id & 0x7FF) != a_id)
    return false;

  // Use first eight bits as index, use the last three bits to set a flag on the index
  flag = (cyg_uint8)(1 << (cyg_uint8)(a_id & 0x7));
  index = (cyg_uint8)((a_id & 0x7F8) >> 3);

  a_chan->m_id_filter[index] &= ~flag;

  return true;
}

// mpc555_add_rtr_id
static bool mpc555_add_rtr_id(can_channel * a_chan, cyg_uint16 a_id)
{
  int idx = 2;
  int free_idx = 0;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF0);

  if((a_id & 0x7FF) != a_id)
    return false;

  while(idx < 15)
  {
    if(a_chan->m_rtr_id[idx] == a_id)
      return true;

    if(a_chan->m_rtr_id[idx] == 0xFFFF)
      free_idx = idx;

    idx++;
  }

  if(!free_idx)
    return false;

  buffer[free_idx].m_code = 0; // Disable it till data are presented
  buffer[free_idx].m_rtr  = 0;
  buffer[free_idx].m_unused = 0;
  buffer[free_idx].m_ide = 0;  // Use standard identifier
  buffer[free_idx].m_id = a_id;
  a_chan->m_rtr_id[free_idx] = a_id;  

  return true;
}

// mpc555_remove_rtr_id
static bool mpc555_remove_rtr_id(can_channel * a_chan, cyg_uint16 a_id)
{
  int idx = 2;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF0);
  
  if((a_id & 0x7FF) != a_id)
    return false;

  while(idx < 15)
  {
    if(a_chan->m_rtr_id[idx] == a_id)
      break;

    idx++;
  }

  if(idx > 14)
    return false;

  buffer[idx].m_code = 0; 
  a_chan->m_rtr_id[idx] = 0xFFFF;  

  return true;
}

// mpc555_update_rtr_buffer
static bool mpc555_update_rtr_buffer(can_channel * a_chan, const can_message * a_msg)
{
  int idx = 2;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF0);

  if((a_msg->m_CAN_id & 0x7FF) != a_msg->m_CAN_id)
    return false;

  while(idx < 15)
  {
    if(a_chan->m_rtr_id[idx] == a_msg->m_CAN_id)
      break;

    idx++;
  }

  if(idx > 14)
    return false;

  buffer[idx].m_length = a_msg->m_DLC;
  buffer[idx].m_rtr = a_msg->m_RTR;
  buffer[idx].m_ide = 0;
  buffer[idx].m_data_byte[0] = a_msg->m_Data[0];
  buffer[idx].m_data_byte[1] = a_msg->m_Data[1];
  buffer[idx].m_data_byte[2] = a_msg->m_Data[2];
  buffer[idx].m_data_byte[3] = a_msg->m_Data[3];
  buffer[idx].m_data_byte[4] = a_msg->m_Data[4];
  buffer[idx].m_data_byte[5] = a_msg->m_Data[5];
  buffer[idx].m_data_byte[6] = a_msg->m_Data[6];
  buffer[idx].m_data_byte[7] = a_msg->m_Data[7];

  buffer[idx].m_code = TOUCAN_MB_TX_REM_REP;

  return true;
}

//----------------------------------
// Private functions implementations
//----------------------------------
// toucan_MB_off
// Set all 16 Toucan Message buffers to inactive and clear the IFLAG register
void toucan_MB_off(cyg_addrword_t a_port)
{
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(a_port + MBUFF0);
  int count;
  cyg_uint16 iflag;

  for(count = 0; count < 16; count++)
  {
    buffer[count].m_code   = 0;  // Make the buffer inactive
    buffer[count].m_ide    = 0;  // Ensure standard identifier allocation
    buffer[count].m_unused = 0;  // These bits must be set to zero when using standard identifiers
  }
 
  // Clear all 16 bits in the iflag register by reading, then writing
  HAL_READ_UINT16(a_port + IFLAG, iflag);
  iflag = 0;
  HAL_WRITE_UINT16(a_port + IFLAG, iflag);
}

// toucan_bus_on
// Unlock all message buffers and activate CAN 2.0B controller
void toucan_bus_on(cyg_addrword_t a_port)
{
  cyg_uint16 tcntimer;
  cyg_uint16 tcnmcr;

  HAL_READ_UINT16(a_port + TIMER, tcntimer); // Unlock all message buffers by reading the TIMER register

  HAL_READ_UINT16(a_port + TCNMCR, tcnmcr);  // Enable the TOUCAN CAN 2.0B controller
  tcnmcr &= ~((cyg_uint16)TCNMCR_HALT);
  HAL_WRITE_UINT16(a_port + TCNMCR, tcnmcr);
}

// Toucan_MB_init
// Initialise the message buffers of the TOUCAN CAN2.0B CAN controller
void toucan_MB_init(cyg_addrword_t a_port)
{
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(a_port + MBUFF0);

  // Make all receive buffers receive everything on the bus
  cyg_uint16 rxmask;
  HAL_READ_UINT16(a_port +  RXGMASKHI, rxmask);
  rxmask &= 0x0008;
  HAL_WRITE_UINT16(a_port + RXGMASKHI, rxmask);
  HAL_READ_UINT16(a_port +  RXGMASKLO, rxmask);
  rxmask &= 0x0000;
  HAL_WRITE_UINT16(a_port + RXGMASKLO, rxmask);
  HAL_READ_UINT16(a_port +  RX14MASKHI, rxmask);
  rxmask &= 0x0008;
  HAL_WRITE_UINT16(a_port + RX14MASKHI, rxmask);
  HAL_READ_UINT16(a_port +  RX14MASKLO, rxmask);
  rxmask &= 0x0000;
  HAL_WRITE_UINT16(a_port + RX14MASKLO, rxmask);
  HAL_READ_UINT16(a_port +  RX15MASKHI, rxmask);
  rxmask &= 0x0008;
  HAL_WRITE_UINT16(a_port + RX15MASKHI, rxmask);
  HAL_READ_UINT16(a_port +  RX15MASKLO, rxmask);
  rxmask &= 0x0000;
  HAL_WRITE_UINT16(a_port + RX15MASKLO, rxmask);

  buffer[0].m_code = TOUCAN_MB_RX_ENABLE;
  buffer[1].m_code = TOUCAN_MB_RX_ENABLE;
}

// toucan_RxMessage
void toucan_RxMessage(t_can_messagebuffer * a_messagebuffer, can_channel * a_channel)
{
  can_message msg;

  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_channel->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  cyg_uint16 estat;
  HAL_READ_UINT16(port + ESTAT, estat);
  estat &= (cyg_uint16)(ESTAT_TXWARN | ESTAT_RXWARN);
  if(estat)
    a_channel->m_error |= E_CAN_ERROR;
  else
    a_channel->m_error &= ~((Cyg_ErrNo)E_CAN_ERROR);

  msg.m_CAN_id  = a_messagebuffer->m_id;
  msg.m_RTR     = a_messagebuffer->m_rtr;
  msg.m_DLC     = a_messagebuffer->m_length;
  msg.m_Data[0] = a_messagebuffer->m_data_byte[0];
  msg.m_Data[1] = a_messagebuffer->m_data_byte[1];
  msg.m_Data[2] = a_messagebuffer->m_data_byte[2];
  msg.m_Data[3] = a_messagebuffer->m_data_byte[3];
  msg.m_Data[4] = a_messagebuffer->m_data_byte[4];
  msg.m_Data[5] = a_messagebuffer->m_data_byte[5];
  msg.m_Data[6] = a_messagebuffer->m_data_byte[6];
  msg.m_Data[7] = a_messagebuffer->m_data_byte[7];

  a_messagebuffer->m_code = TOUCAN_MB_RX_ENABLE; // Ok, the data are copied, re-enable the buffer

  // Pass on the usefull data towards higher layers
  a_channel->m_callbacks->can_rcv_message(a_channel, &msg);
}

// toucan_TxMessage
void toucan_TxMessage(t_can_messagebuffer * a_messagebuffer, const can_message * a_msg, can_channel * a_channel)
{
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)a_channel->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  cyg_uint16 estat;
  HAL_READ_UINT16(port + ESTAT, estat);
  estat &= (cyg_uint16)(ESTAT_TXWARN | ESTAT_RXWARN);
  if(estat)
    a_channel->m_error |= E_CAN_ERROR;
  else
    a_channel->m_error &= ~((Cyg_ErrNo)E_CAN_ERROR);

  a_messagebuffer->m_length       = a_msg->m_DLC;
  a_messagebuffer->m_id           = a_msg->m_CAN_id;
  a_messagebuffer->m_rtr          = a_msg->m_RTR;
  a_messagebuffer->m_ide          = 0; // Use the standard allocation scheme
  a_messagebuffer->m_unused       = 0;
  if(!a_msg->m_RTR)
  {
    a_messagebuffer->m_data_byte[0] = a_msg->m_Data[0];
    a_messagebuffer->m_data_byte[1] = a_msg->m_Data[1];
    a_messagebuffer->m_data_byte[2] = a_msg->m_Data[2];
    a_messagebuffer->m_data_byte[3] = a_msg->m_Data[3];
    a_messagebuffer->m_data_byte[4] = a_msg->m_Data[4];
    a_messagebuffer->m_data_byte[5] = a_msg->m_Data[5];
    a_messagebuffer->m_data_byte[6] = a_msg->m_Data[6];
    a_messagebuffer->m_data_byte[7] = a_msg->m_Data[7];
  }

  // transmit the message
  a_messagebuffer->m_code = TOUCAN_MB_TX_ENABLE;
}

//------------------------
// Interrupt handling code
//------------------------
// can_rx_interrupt
cyg_uint32 can_rx_interrupt(cyg_vector_t a_vector, can_channel * a_chan, t_can_messagebuffer * a_buffer)
{
  cyg_uint8 index;
  cyg_uint8 flag;

  // Allow the buffer to complete the reception
  while(a_buffer->m_code == TOUCAN_MB_RX_BUSY);
  if(a_buffer->m_code == TOUCAN_MB_RX_OVER)
  { // Signal overrun
    a_chan->m_error |= E_CAN_HARD_OVERRUN;
    cyg_drv_interrupt_unmask(a_vector);
    a_buffer->m_code = TOUCAN_MB_RX_ENABLE;
    
    return 0;
  }

  a_buffer->m_code = TOUCAN_MB_RX_DISABLE;

  // Mask the interrupt till the buffer is completely done
  cyg_drv_interrupt_mask(a_vector);
  cyg_drv_interrupt_acknowledge(a_vector);

  // Pass the message through the message filter
  flag = (cyg_uint8)(1 << (cyg_uint8)(a_buffer->m_id & 0x7));
  index = (cyg_uint8)((a_buffer->m_id & 0x7F8) >> 3);
  if(a_chan->m_id_filter[index] & flag)  
  {
    if((a_chan->m_action1.m_action != 0) &&
       (a_chan->m_action1.m_can_id == a_buffer->m_id))
      (a_chan->m_action1.m_action)();
    else if((a_chan->m_action2.m_action != 0) &&
       (a_chan->m_action2.m_can_id == a_buffer->m_id))
      (a_chan->m_action2.m_action)();

    return CYG_ISR_CALL_DSR;
  }
  else
  {
    cyg_drv_interrupt_unmask(a_vector);
    a_buffer->m_code = TOUCAN_MB_RX_ENABLE;
   
    return 0;
  }
}

// Message buffer 15 is the only transmit message buffer. There is no need to use more than one.
// Two buffers transmitting at the same time won't speed up the transmission process.
static cyg_uint32 mpc555_mb15_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF15);
  buffer->m_code = 0; // Disable the buffer. If the transmit was a RTR frame, will be handled
                      // by one of the receive buffers
  cyg_drv_interrupt_mask(a_vector);
  cyg_drv_interrupt_acknowledge(a_vector);

  return CYG_ISR_CALL_DSR;
}

// Message buffer 0 and 1 are the receive message buffers. This means that while one message is
// Being processed. Another one can be received.
static cyg_uint32 mpc555_mb0_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF0);
  
  return can_rx_interrupt(a_vector, chan, buffer);
}

static cyg_uint32 mpc555_mb1_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;
  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF1);

  return can_rx_interrupt(a_vector, chan, buffer);
}

static void mpc555_mb15_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  cyg_drv_interrupt_unmask(a_vector);

  if(!chan->m_transmitting)
    (chan->m_callbacks->can_xmt_message)(chan);
}

static void mpc555_mb0_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF0);
  toucan_RxMessage(buffer, chan);

  cyg_drv_interrupt_unmask(a_vector);
}

static void mpc555_mb1_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  mpc555_can_info * mpc555_chan = (mpc555_can_info *)chan->m_dev_priv;
  cyg_addrword_t port = mpc555_chan->m_base;

  t_can_messagebuffer * buffer = (t_can_messagebuffer *)(port + MBUFF1);
  toucan_RxMessage(buffer, chan);

  cyg_drv_interrupt_unmask(a_vector);
}

// No need doing anything here.
// Re-enable interrupts as fast as possible and handle the situation later on
static cyg_uint32 mpc555_boff_ISR(cyg_vector_t a_vector, cyg_addrword_t a_data)
{
  cyg_drv_interrupt_mask(a_vector);
  cyg_drv_interrupt_acknowledge(a_vector);

  return CYG_ISR_CALL_DSR;
}

// Everyone waiting for data can now be informed that they need no longer wait
// A reset of the controller is the only way out
static void mpc555_boff_DSR(cyg_vector_t a_vector, cyg_ucount32 a_count, cyg_addrword_t a_data)
{
  can_channel * chan = (can_channel *)a_data;
  can_buf_t * can_buf;

  can_buf = &chan->m_in_buf;
  if(can_buf->m_waiting)
  {
    can_buf->m_waiting = false;
    can_buf->m_abort = true;
    cyg_drv_cond_signal(&can_buf->m_wait);
  }

  can_buf = &chan->m_out_buf;
  if(can_buf->m_waiting)
  {
    can_buf->m_waiting = false;
    can_buf->m_abort = true;
    cyg_drv_cond_signal(&can_buf->m_wait);
  }
 
  chan->m_error = E_CAN_BOFF; 
  cyg_drv_interrupt_unmask(a_vector);
}

#endif //ifdef CYGPKG_IO_CAN_MPC555_CME555
#endif // MPC555_CAN_WITH_INTS_C
