#ifndef KSYNC_H
#define KSYNC_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <cyg/kernel/kapi.h>
class KSingleLock;

//-------------------
// Class declarations
//-------------------
// CSyncObject
class KSyncObject
{
  friend class KSingleLock;
  public:
                  KSyncObject() {}
    virtual       ~KSyncObject() {}

  protected:
    virtual bool  lock(unsigned int a_ticks = 0) = 0;
    virtual void  unlock(void) = 0;

    unsigned int  getMark(unsigned int a_ticks);
};

// CMutex
class KMutex : public KSyncObject,
               public cyg_mutex_t
{
  public:
                  KMutex();
    virtual       ~KMutex();

  protected:
    virtual bool  lock(unsigned int a_ticks = 0);
    virtual void  unlock(void);
};

// CSemaphore
class KSemaphore : public KSyncObject,
                   public cyg_sem_t
{
  public:
                  KSemaphore(unsigned int a_initialcount);
    virtual       ~KSemaphore();

  protected:
    virtual bool  lock(unsigned int a_ticks = 0);
    virtual void  unlock(void);
};

// CCondition
class KCondition : public KSyncObject,
                   public cyg_cond_t
{
  public:
                  KCondition();
    virtual       ~KCondition();

    void          signal(void);

  protected:
    virtual bool  lock(unsigned int a_ticks = 0);
    virtual void  unlock(void);

    KMutex        m_mutex;
};

// CSingleLock
class KSingleLock
{
  public:
                  KSingleLock(KSyncObject * a_syncobject);
    virtual       ~KSingleLock();

    bool          lock(unsigned int a_milisecs = 0);
    void          unlock(void);

  private:
    unsigned int  convertToTicks(unsigned int a_timeout);
    KSyncObject * m_syncobj;
};

#endif // KSYNC_H
